@@ -11,6 +11,7 @@ import flask_login
 import jinja2
 import qrcode
 import qrcode.image.pil
+from flask_babel import _, refresh
 from weasyprint import default_url_fetcher, HTML
 
 from .. import logic
@@ -21,6 +22,8 @@ from ..logic.users import get_user
 
 from .markdown_images import IMAGE_FORMATS
 from .objects import get_object_if_current_user_has_read_permissions
+from .utils import custom_format_datetime
+from ..logic.utils import get_translated_text
 
 SECTIONS = {
     'activity_log',
@@ -33,10 +36,15 @@ SECTIONS = {
 
 def create_pdfexport(
         object_ids: typing.Sequence[int],
-        sections: typing.Set[str] = SECTIONS
+        sections: typing.Set[str] = SECTIONS,
+        lang_code: str = 'en'
 ):
     exported_files = {}
 
+    language_id = logic.languages.get_language_by_lang_code(lang_code).id
+    flask.g.override_locale = lang_code
+    refresh()
+
     base_url = flask.url_for('.index', _external=True)
 
     def custom_url_fetcher(url):
@@ -60,7 +68,7 @@ def create_pdfexport(
                 object_id = int(object_id)
                 file_id = int(file_id)
                 file = exported_files[(object_id, file_id)]
-                if file.storage == 'local' and not file.is_hidden:
+                if file.storage in {'local', 'database'} and not file.is_hidden:
                     for file_extension in IMAGE_FORMATS:
                         if file.original_file_name.endswith(file_extension):
                             image_data = file.open(read_only=True).read()
@@ -84,60 +92,61 @@ def create_pdfexport(
                 user_id = object_log_entry.user_id
                 user_url = jinja2.escape(flask.url_for('.user_profile', user_id=user_id, _external=True))
                 user_name = jinja2.escape(get_user(object_log_entry.user_id).name)
-                entry_datetime = jinja2.escape(object_log_entry.utc_datetime.strftime('%Y-%m-%d %H:%M'))
-                text = f'{entry_datetime} — <a href="{user_url}">{user_name} (#{user_id})</a>'
+
+                entry_datetime = jinja2.escape(custom_format_datetime(object_log_entry.utc_datetime))
+                text = f'{entry_datetime} — '
                 if object_log_entry.type == ObjectLogEntryType.CREATE_BATCH:
-                    text += ' created this object as part of a batch.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> created this object as part of a batch.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.CREATE_OBJECT:
-                    text += ' created this object.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> created this object.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.EDIT_OBJECT:
-                    text += ' edited this object.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> edited this object.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.POST_COMMENT:
-                    text += ' posted a comment.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> posted a comment.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.RESTORE_OBJECT_VERSION:
-                    text += ' restored a previous version of this object.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> restored a previous version of this object.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.UPLOAD_FILE:
-                    text += ' posted a file.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> posted a file.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.USE_OBJECT_IN_MEASUREMENT:
                     try:
                         measurement_id = int(object_log_entry.data['measurement_id'])
                         object_url = jinja2.escape(flask.url_for('.object', object_id=measurement_id, _external=True))
                         permissions = logic.object_permissions.get_user_object_permissions(measurement_id, flask_login.current_user.id)
                         if logic.object_permissions.Permissions.READ in permissions:
-                            measurement_name = jinja2.escape(get_object(measurement_id).data['name']['text'])
-                            text += f' used this object in <a href="{object_url}">measurement {measurement_name} (#{measurement_id})</a>.'
+                            measurement_name = jinja2.escape(get_translated_text(get_object(measurement_id).data['name']['text']))
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object in <a href="%(object_url)s">measurement %(measurement_name)s (#%(measurement_id)s)</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, measurement_id=measurement_id, measurement_name=measurement_name)
                         else:
-                            text += f' used this object in <a href="{object_url}">measurement #{measurement_id}</a>.'
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object in <a href="%(object_url)s">measurement #%(measurement_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, measurement_id=measurement_id)
                     except Exception:
-                        text += ' used this object in a measurement.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object in a measurement.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.USE_OBJECT_IN_SAMPLE_CREATION:
                     try:
                         sample_id = int(object_log_entry.data['sample_id'])
                         object_url = jinja2.escape(flask.url_for('.object', object_id=sample_id, _external=True))
                         permissions = logic.object_permissions.get_user_object_permissions(sample_id, flask_login.current_user.id)
                         if logic.object_permissions.Permissions.READ in permissions:
-                            sample_name = jinja2.escape(get_object(sample_id).data['name']['text'])
-                            text += f' used this object to create <a href="{object_url}">sample {sample_name} (#{sample_id})</a>.'
+                            sample_name = jinja2.escape(get_translated_text(get_object(sample_id).data['name']['text']))
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object to create <a href="%(object_url)s">sample %(sample_name)s (#%(sample_id)s)</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, sample_id=sample_id, sample_name=sample_name)
                         else:
-                            text += f' used this object to create <a href="{object_url}">sample #{sample_id}</a>.'
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object to create <a href="%(object_url)s">sample #%(sample_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, sample_id=sample_id)
                     except Exception:
-                        text += ' used this object to create a sample.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> used this object to create a sample.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.ASSIGN_LOCATION:
                     object_location_assignment_id = object_log_entry.data['object_location_assignment_id']
                     object_location_assignment = logic.locations.get_object_location_assignment(object_location_assignment_id)
                     if object_location_assignment.location_id is not None and object_location_assignment.responsible_user_id is not None:
-                        user_url = jinja2.escape(flask.url_for('.user_profile', user_id=object_location_assignment.responsible_user_id, _external=True))
+                        other_user_url = jinja2.escape(flask.url_for('.user_profile', user_id=object_location_assignment.responsible_user_id, _external=True))
                         location_url = jinja2.escape(flask.url_for('.location', location_id=object_location_assignment.location_id, _external=True))
-                        text += f' assigned this object to <a href="{location_url}">location #{object_location_assignment.location_id}</a> and <a href="{user_url}">user #{object_location_assignment.responsible_user_id}</a>.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> assigned this object to <a href="%(location_url)s">location #%(location_id)s</a> and <a href="%(other_user_url)s">user #%(responsible_user_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, location_url=location_url, location_id=object_location_assignment.location_id, other_user_url=other_user_url, responsible_user_id=object_location_assignment.responsible_user_id)
                     elif object_location_assignment.location_id is not None:
                         location_url = jinja2.escape(flask.url_for('.location', location_id=object_location_assignment.location_id, _external=True))
-                        text += f' assigned this object to <a href="{location_url}">location #{object_location_assignment.location_id}</a>.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> assigned this object to <a href="%(location_url)s">location #%(location_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, location_url=location_url, location_id=object_location_assignment.location_id)
                     elif object_location_assignment.responsible_user_id is not None:
-                        user_url = jinja2.escape(flask.url_for('.user_profile', user_id=object_location_assignment.responsible_user_id, _external=True))
-                        text += f' assigned this object to <a href="{user_url}">user #{object_location_assignment.responsible_user_id}</a>.'
+                        other_user_url = jinja2.escape(flask.url_for('.user_profile', user_id=object_location_assignment.responsible_user_id, _external=True))
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> assigned this object to <a href="%(other_user_url)s">user #%(responsible_user_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, other_user_url=other_user_url, responsible_user_id=object_location_assignment.responsible_user_id)
                 elif object_log_entry.type == ObjectLogEntryType.LINK_PUBLICATION:
                     doi = jinja2.escape(object_log_entry.data['doi'])
-                    text += f' linked publication <a href="https://dx.doi.org/{doi}">{doi}</a> to this object.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> linked publication <a href="https://dx.doi.org/%(doi)s">%(doi)s</a> to this object.', user_url=user_url, user_name=user_name, user_id=user_id, doi=doi)
                 elif object_log_entry.type == ObjectLogEntryType.REFERENCE_OBJECT_IN_METADATA:
                     if object_log_entry.data['object_id'] is not None:
                         try:
@@ -146,35 +155,35 @@ def create_pdfexport(
                             permissions = logic.object_permissions.get_user_object_permissions(object_log_entry.data['object_id'], flask_login.current_user.id)
                             if logic.object_permissions.Permissions.READ in permissions:
                                 object_name = jinja2.escape(get_object(object_log_entry.data['object_id']).data['name']['text'])
-                                text += f' referenced this object in the metadata of <a href="{object_url}">object {object_name} (#{other_object_id})</a>.'
+                                text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> referenced this object in the metadata of <a href="%(object_url)s">object %(object_name)s (#%(other_object_id)s)</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, object_name=object_name, other_object_id=other_object_id)
                             else:
-                                text += f' referenced this object in the metadata of <a href="{object_url}">object #{other_object_id}</a>.'
+                                text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> referenced this object in the metadata of <a href="%(object_url)s">object #%(other_object_id)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, object_url=object_url, other_object_id=other_object_id)
                         except Exception:
-                            text += ' referenced this object in the metadata of an unknown object.'
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> referenced this object in the metadata of an unknown object.', user_url=user_url, user_name=user_name, user_id=user_id)
                     else:
-                        text += ' referenced this object in the metadata of another object.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> referenced this object in the metadata of another object.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.EXPORT_TO_DATAVERSE:
                     dataverse_url = jinja2.escape(object_log_entry.data['dataverse_url'])
-                    text += f' exported this object to dataverse as <a href="{dataverse_url}">{dataverse_url}</a>.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> exported this object to dataverse as <a href="%(dataverse_url)s">%(dataverse_url)s</a>.', user_url=user_url, user_name=user_name, user_id=user_id, dataverse_url=dataverse_url)
                 elif object_log_entry.type == ObjectLogEntryType.LINK_PROJECT:
                     try:
                         project = logic.projects.get_project(object_log_entry.data['project_id'])
                         project_url = flask.url_for('.project', project_id=project.id)
-                        text += f' linked this object to <a href="{project_url}">project group {project.name} (#{project.id})</a>.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> linked this object to <a href="%(project_url)s">project group %(project_name)s (#%(project_id)s)</a>.', user_url=user_url, user_name=user_name, user_id=user_id, project_url=project_url, project_name=get_translated_text(project.name), project_id=project.id)
                     except logic.errors.ProjectDoesNotExistError:
-                        text += ' linked this object to a project group.'
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> linked this object to a project group.', user_url=user_url, user_name=user_name, user_id=user_id)
                 elif object_log_entry.type == ObjectLogEntryType.UNLINK_PROJECT:
-                    if object_log_entry.data['project_deleted']:
-                        text += ' deleted the project group this object was linked to.'
+                    if object_log_entry.data.get('project_deleted'):
+                        text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> deleted the project group this object was linked to.', user_url=user_url, user_name=user_name, user_id=user_id)
                     else:
                         try:
                             project = logic.projects.get_project(object_log_entry.data['project_id'])
                             project_url = flask.url_for('.project', project_id=project.id)
-                            text += f' removed the link of this object to <a href="{project_url}">project group {project.name} (#{project.id})</a>.'
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> removed the link of this object to <a href="%(project_url)s">project group %(project_name)s (#%(project_id)s)</a>.', user_url=user_url, user_name=user_name, user_id=user_id, project_url=project_url, project_name=get_translated_text(project.name), project_id=project.id)
                         except logic.errors.ProjectDoesNotExistError:
-                            text += ' removed the link of this object to a project group.'
+                            text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> removed the link of this object to a project group.', user_url=user_url, user_name=user_name, user_id=user_id)
                 else:
-                    text += ' performed an unknown action.'
+                    text += _('<a href="%(user_url)s">%(user_name)s (#%(user_id)s)</a> performed an unknown action.', user_url=user_url, user_name=user_name, user_id=user_id)
                 activity_log_entries.append(text)
 
         locations_entries = []
@@ -229,14 +238,27 @@ def create_pdfexport(
         except logic.errors.UserDoesNotExistError:
             return None
 
+    def get_object_type_name(action):
+        return logic.action_type_translations.get_action_type_translation_for_action_type_in_language(
+            action_type_id=action.type_id,
+            language_id=language_id,
+            use_fallback=True
+        ).object_name
+
     html = flask.render_template(
         'pdfexport/export.html',
-        export_date=datetime.datetime.utcnow().strftime('%Y-%m-%d'),
+        get_object_type_name=get_object_type_name,
+        export_date=datetime.datetime.utcnow(),
         get_object_if_current_user_has_read_permissions=get_object_if_current_user_has_read_permissions,
         objects=objects,
-        get_user=get_user_if_exists
+        get_user=get_user_if_exists,
+        metadata_language=lang_code
     )
 
+    # use regular user language again
+    delattr(flask.g, 'override_locale')
+    refresh()
+
     return HTML(
         string=html,
         url_fetcher=custom_url_fetcher,
