@@ -5,13 +5,18 @@
 
 import flask
 import flask_login
+import json
+from flask_babel import _
 
 from . import frontend
 from .. import logic
 from ..logic.object_permissions import Permissions
 from ..logic.security_tokens import verify_token
+from ..logic.languages import get_languages, get_language, get_language_by_lang_code
+from ..models.languages import Language
 from .projects_forms import CreateProjectForm, EditProjectForm, LeaveProjectForm, InviteUserToProjectForm, InviteGroupToProjectForm, ProjectPermissionsForm, AddSubprojectForm, RemoveSubprojectForm, DeleteProjectForm, RemoveProjectMemberForm, RemoveProjectGroupForm, ObjectLinkForm
 from .utils import check_current_user_is_not_readonly
+from ..logic.utils import get_translated_text
 
 
 @frontend.route('/projects/<int:project_id>', methods=['GET', 'POST'])
@@ -22,24 +27,24 @@ def project(project_id):
         expiration_time_limit = flask.current_app.config['INVITATION_TIME_LIMIT']
         token_data = verify_token(token, salt='invite_to_project', secret_key=flask.current_app.config['SECRET_KEY'], expiration=expiration_time_limit)
         if token_data is None:
-            flask.flash('Invalid project group invitation token. Please request a new invitation.', 'error')
+            flask.flash(_('Invalid project group invitation token. Please request a new invitation.'), 'error')
             return flask.abort(403)
         if 'invitation_id' in token_data:
             if logic.projects.get_project_invitation(token_data['invitation_id']).accepted:
-                flask.flash('This invitation token has already been used. Please request a new invitation.', 'error')
+                flask.flash(_('This invitation token has already been used. Please request a new invitation.'), 'error')
                 return flask.abort(403)
         if token_data.get('project_id', None) != project_id:
             return flask.abort(403)
         permissions = Permissions.from_value(token_data.get('permissions', Permissions.READ.value))
         if permissions == Permissions.NONE:
-            flask.flash('Invalid permissions in project group invitation token. Please request a new invitation.', 'error')
+            flask.flash(_('Invalid permissions in project group invitation token. Please request a new invitation.'), 'error')
             return flask.abort(403)
         user_id = token_data.get('user_id', None)
         if user_id != flask_login.current_user.id:
             if user_id is not None:
                 try:
                     invited_user = logic.users.get_user(user_id)
-                    flask.flash('Please sign in as user "{}" to accept this invitation.'.format(invited_user.name), 'error')
+                    flask.flash(_('Please sign in as user "%(user_name)s" to accept this invitation.', user_name=invited_user.name), 'error')
                 except logic.errors.UserDoesNotExistError:
                     pass
             return flask.abort(403)
@@ -54,9 +59,13 @@ def project(project_id):
         try:
             logic.projects.add_user_to_project(project_id, user_id, permissions, other_project_ids=other_project_ids)
         except logic.errors.UserAlreadyMemberOfProjectError:
-            flask.flash('You are already a member of this project group.', 'error')
+            flask.flash(_('You are already a member of this project group.'), 'error')
         except logic.errors.ProjectDoesNotExistError:
             pass
+    allowed_language_ids = [
+        language.id
+        for language in get_languages(only_enabled_for_input=True)
+    ]
     user_id = flask_login.current_user.id
     try:
         project = logic.projects.get_project(project_id)
@@ -71,15 +80,44 @@ def project(project_id):
         leave_project_form = LeaveProjectForm()
     else:
         leave_project_form = None
+    translations = []
+    name_language_ids = []
+    description_language_ids = []
     if Permissions.WRITE in user_permissions:
         edit_project_form = EditProjectForm()
-        if edit_project_form.name.data is None:
-            edit_project_form.name.data = project.name
-        if edit_project_form.description.data is None:
-            edit_project_form.description.data = project.description
+        for name in project.name.items():
+            lang_id = get_language_by_lang_code(name[0]).id
+            name_language_ids.append(lang_id)
+            item = {
+                'language_id': lang_id,
+                'lang_name': get_translated_text(get_language(lang_id).names),
+                'name': name[1],
+                'description': ''
+            }
+            translations.append(item)
+
+        for description in project.description.items():
+            if description[0] == '':
+                continue
+            lang_id = get_language_by_lang_code(description[0]).id
+            description_language_ids.append(lang_id)
+            for translation in translations:
+                if lang_id == translation['language_id']:
+                    translation['description'] = description[1]
+                    break
+            else:
+                item = {
+                    'language_id': lang_id,
+                    'lang_name': get_translated_text(get_language(lang_id).names),
+                    'name': '',
+                    'description': description[1]
+                }
+                translations.append(item)
     else:
         edit_project_form = None
     show_edit_form = False
+    english = get_language(Language.ENGLISH)
+
     project_member_user_ids_and_permissions = logic.projects.get_project_member_user_ids_and_permissions(project_id=project_id, include_groups=False)
     project_member_group_ids_and_permissions = logic.projects.get_project_member_group_ids_and_permissions(project_id=project_id)
 
@@ -87,7 +125,7 @@ def project(project_id):
     project_member_user_ids.sort(key=lambda user_id: logic.users.get_user(user_id).name.lower())
 
     project_member_group_ids = list(project_member_group_ids_and_permissions.keys())
-    project_member_group_ids.sort(key=lambda group_id: logic.groups.get_group(group_id).name.lower())
+    project_member_group_ids.sort(key=lambda group_id: get_translated_text(logic.groups.get_group(group_id).name).lower())
 
     if Permissions.GRANT in user_permissions:
         invitable_user_list = []
@@ -160,18 +198,18 @@ def project(project_id):
             try:
                 logic.projects.remove_user_from_project(project_id=project_id, user_id=user_id)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.UserDoesNotExistError:
                 return flask.abort(500)
             except logic.errors.UserNotMemberOfProjectError:
-                flask.flash('You have already left the project group.', 'error')
+                flask.flash(_('You have already left the project group.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.NoMemberWithGrantPermissionsForProjectError:
-                flask.flash('You cannot leave this project group, because your are the only user with GRANT permissions.', 'error')
+                flask.flash(_('You cannot leave this project group, because your are the only user with GRANT permissions.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             else:
-                flask.flash('You have successfully left the project group.', 'success')
+                flask.flash(_('You have successfully left the project group.'), 'success')
                 # create object log entry if this caused the deletion of a project linked to an object
                 try:
                     logic.projects.get_project(project_id)
@@ -198,10 +236,10 @@ def project(project_id):
             try:
                 logic.projects.delete_project(project_id=project_id)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group has already been deleted.', 'success')
+                flask.flash(_('This project group has already been deleted.'), 'success')
                 return flask.redirect(flask.url_for('.projects'))
             else:
-                flask.flash('You have successfully deleted the project group.', 'success')
+                flask.flash(_('You have successfully deleted the project group.'), 'success')
                 return flask.redirect(flask.url_for('.projects'))
     if 'remove_member' in flask.request.form and Permissions.GRANT in user_permissions:
         if remove_project_member_form.validate_on_submit():
@@ -210,27 +248,27 @@ def project(project_id):
             try:
                 member_id = int(member_id_str)
             except ValueError:
-                flask.flash('The member ID was invalid. Please contact an administrator.', 'error')
+                flask.flash(_('The member ID was invalid. Please contact an administrator.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             if member_id == flask_login.current_user.id:
-                flask.flash('You cannot remove yourself from a project group. Please press "Leave Project Group" instead.', 'error')
+                flask.flash(_('You cannot remove yourself from a project group. Please press "Leave Project Group" instead.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             try:
                 logic.projects.remove_user_from_project(project_id=project_id, user_id=member_id)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.UserDoesNotExistError:
-                flask.flash('This user does not exist.', 'error')
+                flask.flash(_('This user does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             except logic.errors.UserNotMemberOfProjectError:
-                flask.flash('This user is not a member of this project group.', 'error')
+                flask.flash(_('This user is not a member of this project group.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             except logic.errors.NoMemberWithGrantPermissionsForProjectError:
-                flask.flash('You cannot remove this user from this project group, because they are the only user with GRANT permissions.', 'error')
+                flask.flash(_('You cannot remove this user from this project group, because they are the only user with GRANT permissions.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             else:
-                flask.flash('You have successfully removed this user from the project group.', 'success')
+                flask.flash(_('You have successfully removed this user from the project group.'), 'success')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
     if 'remove_group' in flask.request.form and Permissions.GRANT in user_permissions:
         if remove_project_group_form.validate_on_submit():
@@ -239,50 +277,80 @@ def project(project_id):
             try:
                 group_id = int(group_id_str)
             except ValueError:
-                flask.flash('The basic group ID was invalid. Please contact an administrator.', 'error')
+                flask.flash(_('The basic group ID was invalid. Please contact an administrator.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             try:
                 logic.projects.remove_group_from_project(project_id=project_id, group_id=group_id)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.GroupDoesNotExistError:
-                flask.flash('This basic group does not exist.', 'error')
+                flask.flash(_('This basic group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             except logic.errors.GroupNotMemberOfProjectError:
-                flask.flash('This basic group is not a member of this project group.', 'error')
+                flask.flash(_('This basic group is not a member of this project group.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             except logic.errors.NoMemberWithGrantPermissionsForProjectError:
-                flask.flash('You cannot remove this basic group from this project group, because they are the only basic group with GRANT permissions.', 'error')
+                flask.flash(_('You cannot remove this basic group from this project group, because they are the only basic group with GRANT permissions.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             else:
-                flask.flash('You have successfully removed this basic group from the project group.', 'success')
+                flask.flash(_('You have successfully removed this basic group from the project group.'), 'success')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
     if 'edit' in flask.request.form and Permissions.WRITE in user_permissions:
         show_edit_form = True
         if edit_project_form.validate_on_submit():
             check_current_user_is_not_readonly()
             try:
-                logic.projects.update_project(project_id, edit_project_form.name.data, edit_project_form.description.data)
+                translations = json.loads(edit_project_form.translations.data)
+                if not translations:
+                    raise ValueError(_('Please enter at least an english name.'))
+                names = {}
+                descriptions = {}
+                for translation in translations:
+                    name = translation['name'].strip()
+                    description = translation['description'].strip()
+                    language_id = int(translation['language_id'])
+
+                    if language_id not in allowed_language_ids:
+                        continue
+
+                    if language_id == Language.ENGLISH:
+                        if name == '':
+                            raise ValueError(_('Please enter at least an english name.'))
+                    elif name == '' and description == '':
+                        continue
+
+                    lang_code = get_language(language_id).lang_code
+
+                    names[lang_code] = name
+                    if description != '':
+                        descriptions[lang_code] = description
+                    else:
+                        descriptions[lang_code] = ''
+
+                logic.projects.update_project(project_id, names, descriptions)
+            except ValueError as e:
+                flask.flash(str(e), 'error')
+                edit_project_form.translations.errors.append(str(e))
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.ProjectAlreadyExistsError:
-                edit_project_form.name.errors.append('A project group with this name already exists.')
+                edit_project_form.name.errors.append(_('A project group with this name already exists.'))
             except logic.errors.InvalidProjectNameError:
-                edit_project_form.name.errors.append('This project group name is invalid.')
+                edit_project_form.name.errors.append(_('This project group name is invalid.'))
             else:
-                flask.flash('Project group information updated successfully.', 'success')
+                flask.flash(_('Project group information updated successfully.'), 'success')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
     if 'add_user' in flask.request.form and Permissions.GRANT in user_permissions:
         if invite_user_form.validate_on_submit():
             check_current_user_is_not_readonly()
             if not any(user.id == invite_user_form.user_id.data for user in invitable_user_list):
-                flask.flash('You cannot add this user.', 'error')
+                flask.flash(_('You cannot add this user.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             permissions = Permissions.from_value(invite_user_form.permissions.data)
             if Permissions.READ not in permissions:
-                flask.flash('Please select read permissions (or higher) for the invitation.', 'error')
+                flask.flash(_('Please select read permissions (or higher) for the invitation.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             try:
                 other_project_ids = []
@@ -294,32 +362,32 @@ def project(project_id):
                         pass
                 logic.projects.invite_user_to_project(project_id, invite_user_form.user_id.data, flask_login.current_user.id, other_project_ids, permissions)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.UserDoesNotExistError:
-                flask.flash('This user does not exist.', 'error')
+                flask.flash(_('This user does not exist.'), 'error')
             except logic.errors.UserAlreadyMemberOfProjectError:
-                flask.flash('This user is already a member of this project group.', 'error')
+                flask.flash(_('This user is already a member of this project group.'), 'error')
             else:
-                flask.flash('The user was successfully invited to the project group.', 'success')
+                flask.flash(_('The user was successfully invited to the project group.'), 'success')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
     if 'add_group' in flask.request.form and Permissions.GRANT in user_permissions:
         if invite_group_form.validate_on_submit():
             check_current_user_is_not_readonly()
             if not any(group.id == invite_group_form.group_id.data for group in invitable_group_list):
-                flask.flash('You cannot add this basic group.', 'error')
+                flask.flash(_('You cannot add this basic group.'), 'error')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
             try:
                 logic.projects.add_group_to_project(project_id, invite_group_form.group_id.data, permissions=Permissions.READ)
             except logic.errors.ProjectDoesNotExistError:
-                flask.flash('This project group does not exist.', 'error')
+                flask.flash(_('This project group does not exist.'), 'error')
                 return flask.redirect(flask.url_for('.projects'))
             except logic.errors.GroupDoesNotExistError:
-                flask.flash('This basic group does not exist.', 'error')
+                flask.flash(_('This basic group does not exist.'), 'error')
             except logic.errors.GroupAlreadyMemberOfProjectError:
-                flask.flash('This basic group is already a member of this project group.', 'error')
+                flask.flash(_('This basic group is already a member of this project group.'), 'error')
             else:
-                flask.flash('The basic group was successfully added to the project group.', 'success')
+                flask.flash(_('The basic group was successfully added to the project group.'), 'success')
                 return flask.redirect(flask.url_for('.project', project_id=project_id))
     if not flask.current_app.config['DISABLE_SUBPROJECTS']:
         if 'remove_subproject' in flask.request.form and Permissions.GRANT in user_permissions:
@@ -327,20 +395,20 @@ def project(project_id):
                 check_current_user_is_not_readonly()
                 child_project_id = remove_subproject_form.child_project_id.data
                 if child_project_id not in child_project_ids:
-                    flask.flash('Project group #{} is not a child of this project group.'.format(int(child_project_id)), 'error')
+                    flask.flash(_('Project group #%(child_project_id)s is not a child of this project group.', child_project_id=int(child_project_id)), 'error')
                 else:
                     logic.projects.delete_subproject_relationship(project_id, child_project_id)
-                    flask.flash('The child project group was successfully removed from this project group.', 'success')
+                    flask.flash(_('The child project group was successfully removed from this project group.'), 'success')
                     return flask.redirect(flask.url_for('.project', project_id=project_id))
         if 'add_subproject' in flask.request.form and Permissions.GRANT in user_permissions:
             if add_subproject_form is not None and add_subproject_form.validate_on_submit():
                 check_current_user_is_not_readonly()
                 child_project_id = add_subproject_form.child_project_id.data
                 if child_project_id not in addable_project_ids:
-                    flask.flash('Project group #{} cannot become a child of this project group.'.format(int(child_project_id)), 'error')
+                    flask.flash(_('Project group #%(child_project_id)s cannot become a child of this project group.', child_project_id=int(child_project_id)), 'error')
                 else:
                     logic.projects.create_subproject_relationship(project_id, child_project_id, child_can_add_users_to_parent=add_subproject_form.child_can_add_users_to_parent.data)
-                    flask.flash('The child project group was successfully added to this project group.', 'success')
+                    flask.flash(_('The child project group was successfully added to this project group.'), 'success')
                     return flask.redirect(flask.url_for('.project', project_id=project_id))
     object_id = object.id if object is not None else None
     object_action = None
@@ -361,7 +429,7 @@ def project(project_id):
                     if not flask.current_app.config['LOAD_OBJECTS_IN_BACKGROUND']:
                         for object_info in logic.object_permissions.get_object_info_with_permissions(user_id, Permissions.GRANT, action_type_id=action_type.id):
                             if object_info.object_id not in already_linked_object_ids:
-                                linkable_objects.append((object_info.object_id, object_info.name_text))
+                                linkable_objects.append((object_info.object_id, get_translated_text(object_info.name_json)))
     if object is not None:
         object_permissions = logic.object_permissions.get_user_object_permissions(object.object_id, flask_login.current_user.id)
         if Permissions.READ in object_permissions:
@@ -370,6 +438,11 @@ def project(project_id):
             object = None
     return flask.render_template(
         'projects/project.html',
+        ENGLISH=english,
+        translations=translations,
+        languages=get_languages(only_enabled_for_input=True),
+        name_language_ids=name_language_ids,
+        description_language_ids=description_language_ids,
         get_user=logic.users.get_user,
         get_group=logic.groups.get_group,
         get_project=logic.projects.get_project,
@@ -404,7 +477,7 @@ def project(project_id):
         add_subproject_form=add_subproject_form,
         addable_projects=addable_projects,
         remove_subproject_form=remove_subproject_form,
-        user_may_edit_permissions=Permissions.GRANT in user_permissions
+        user_may_edit_permissions=Permissions.GRANT in user_permissions,
     )
 
 
@@ -426,42 +499,71 @@ def projects():
     for project in projects:
         project.permissions = logic.projects.get_user_project_permissions(project_id=project.id, user_id=flask_login.current_user.id, include_groups=True)
     create_project_form = CreateProjectForm()
-    if create_project_form.name.data is None:
-        create_project_form.name.data = ''
-    if create_project_form.description.data is None:
-        create_project_form.description.data = ''
     show_create_form = False
     if 'create' in flask.request.form:
+        allowed_language_ids = [
+            language.id
+            for language in get_languages(only_enabled_for_input=True)
+        ]
         show_create_form = True
         if create_project_form.validate_on_submit():
             check_current_user_is_not_readonly()
             if flask_login.current_user.is_admin or not flask.current_app.config['ONLY_ADMINS_CAN_CREATE_PROJECTS']:
                 try:
-                    project_id = logic.projects.create_project(create_project_form.name.data, create_project_form.description.data, flask_login.current_user.id).id
+                    translations = json.loads(create_project_form.translations.data)
+                    if not translations:
+                        raise ValueError(_('Please enter at least an english name.'))
+                    names = {}
+                    descriptions = {}
+                    for translation in translations:
+                        name = translation['name'].strip()
+                        description = translation['description'].strip()
+                        language_id = int(translation['language_id'])
+
+                        if language_id not in allowed_language_ids:
+                            continue
+
+                        if language_id == Language.ENGLISH:
+                            if name == '':
+                                raise ValueError(_('Please enter at least an english name.'))
+
+                        lang_code = get_language(language_id).lang_code
+
+                        names[lang_code] = name
+                        if description != '':
+                            descriptions[lang_code] = description
+                        else:
+                            descriptions[lang_code] = ''
+
+                    project_id = logic.projects.create_project(names, descriptions, flask_login.current_user.id).id
+                except ValueError as e:
+                    flask.flash(str(e), 'error')
+                    create_project_form.translations.errors.append(str(e))
                 except logic.errors.ProjectAlreadyExistsError:
-                    create_project_form.name.errors.append('A project group with this name already exists.')
+                    create_project_form.translations.errors.append(_('A project group with this name already exists.'))
                 except logic.errors.InvalidProjectNameError:
-                    create_project_form.name.errors.append('This project group name is invalid.')
+                    create_project_form.translations.errors.append(_('This project group name is invalid.'))
                 else:
-                    flask.flash('The project group has been created successfully.', 'success')
+                    flask.flash(_('The project group has been created successfully.'), 'success')
                     return flask.redirect(flask.url_for('.project', project_id=project_id))
             else:
-                create_project_form.name.errors.append('Only administrators can create project groups.')
+                create_project_form.translations.errors.append(_('Only administrators can create project groups.'))
 
     projects_by_id = {
         project.id: project
         for project in projects
     }
-
     project_id_hierarchy_list = logic.projects.get_project_id_hierarchy_list(list(projects_by_id))
-
+    english = get_language(Language.ENGLISH)
     return flask.render_template(
         "projects/projects.html",
         create_project_form=create_project_form,
         show_create_form=show_create_form,
         Permissions=logic.projects.Permissions,
         projects_by_id=projects_by_id,
-        project_id_hierarchy_list=project_id_hierarchy_list
+        project_id_hierarchy_list=project_id_hierarchy_list,
+        languages=get_languages(only_enabled_for_input=True),
+        ENGLISH=english
     )
 
 
@@ -524,9 +626,9 @@ def update_project_permissions(project_id):
                 continue
             permissions = Permissions.from_name(group_permissions_data['permissions'])
             logic.projects.update_group_project_permissions(project_id=project_id, group_id=group_id, permissions=permissions)
-        flask.flash("Successfully updated project group permissions.", 'success')
+        flask.flash(_("Successfully updated project group permissions."), 'success')
     else:
-        flask.flash("A problem occurred while changing the project group permissions. Please try again.", 'error')
+        flask.flash(_("A problem occurred while changing the project group permissions. Please try again."), 'error')
     try:
         logic.projects.get_project(project_id)
     except logic.errors.ProjectDoesNotExistError:
@@ -540,27 +642,27 @@ def link_object(project_id):
     check_current_user_is_not_readonly()
     object_link_form = ObjectLinkForm()
     if not object_link_form.validate_on_submit():
-        flask.flash("Missing or invalid object ID.", 'error')
+        flask.flash(_("Missing or invalid object ID."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     object_id = object_link_form.object_id.data
     try:
         if Permissions.GRANT not in logic.projects.get_user_project_permissions(project_id, flask_login.current_user.id, True):
-            flask.flash("You do not have GRANT permissions for this project group.", 'error')
+            flask.flash(_("You do not have GRANT permissions for this project group."), 'error')
             return flask.redirect(flask.url_for('.project', project_id=project_id))
         if Permissions.GRANT not in logic.object_permissions.get_user_object_permissions(object_id, flask_login.current_user.id):
-            flask.flash("You do not have GRANT permissions for this object.", 'error')
+            flask.flash(_("You do not have GRANT permissions for this object."), 'error')
             return flask.redirect(flask.url_for('.project', project_id=project_id))
         logic.projects.link_project_and_object(project_id, object_id, flask_login.current_user.id)
     except logic.errors.ProjectObjectLinkAlreadyExistsError:
-        flask.flash("Project group is already linked to an object or object is already linked to a project group.", 'error')
+        flask.flash(_("Project group is already linked to an object or object is already linked to a project group."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     except logic.errors.ProjectDoesNotExistError:
-        flask.flash("Project group does not exist.", 'error')
+        flask.flash(_("Project group does not exist."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     except logic.errors.ObjectDoesNotExistError:
-        flask.flash("Object does not exist.", 'error')
+        flask.flash(_("Object does not exist."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
-    flask.flash("Successfully linked the object to a project group.", 'success')
+    flask.flash(_("Successfully linked the object to a project group."), 'success')
     return flask.redirect(flask.url_for('.project', project_id=project_id))
 
 
@@ -570,22 +672,22 @@ def unlink_object(project_id):
     check_current_user_is_not_readonly()
     object_link_form = ObjectLinkForm()
     if not object_link_form.validate_on_submit():
-        flask.flash("Missing or invalid object ID.", 'error')
+        flask.flash(_("Missing or invalid object ID."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     object_id = object_link_form.object_id.data
     try:
         if Permissions.GRANT not in logic.projects.get_user_project_permissions(project_id, flask_login.current_user.id, True):
-            flask.flash("You do not have GRANT permissions for this project group.", 'error')
+            flask.flash(_("You do not have GRANT permissions for this project group."), 'error')
             return flask.redirect(flask.url_for('.project', project_id=project_id))
         logic.projects.unlink_project_and_object(project_id, object_id, flask_login.current_user.id)
     except logic.errors.ProjectObjectLinkDoesNotExistsError:
-        flask.flash("No link exists between this object and project group.", 'error')
+        flask.flash(_("No link exists between this object and project group."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     except logic.errors.ProjectDoesNotExistError:
-        flask.flash("Project group does not exist.", 'error')
+        flask.flash(_("Project group does not exist."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
     except logic.errors.ObjectDoesNotExistError:
-        flask.flash("Object does not exist.", 'error')
+        flask.flash(_("Object does not exist."), 'error')
         return flask.redirect(flask.url_for('.project', project_id=project_id))
-    flask.flash("Successfully unlinked the object and project group.", 'success')
+    flask.flash(_("Successfully unlinked the object and project group."), 'success')
     return flask.redirect(flask.url_for('.project', project_id=project_id))
