@@ -6,17 +6,15 @@
 
 from flask_wtf import FlaskForm
 from wtforms import StringField, IntegerField
-from wtforms.validators import InputRequired, Length
+from wtforms.validators import InputRequired, Length, DataRequired
 
 
 class EditGroupForm(FlaskForm):
-    name = StringField(validators=[Length(min=1, max=100)])
-    description = StringField()
+    translations = StringField(validators=[DataRequired()])
 
 
 class CreateGroupForm(FlaskForm):
-    name = StringField(validators=[Length(min=1, max=100)])
-    description = StringField()
+    translations = StringField(validators=[DataRequired()])
 
 
 class LeaveGroupForm(FlaskForm):
@@ -53,3 +51,8 @@ class OtherSettingsForm(FlaskForm):
 
 class CreateAPITokenForm(FlaskForm):
     description = StringField('description', validators=[Length(min=1, max=100)])
+
+
+class ManageTwoFactorAuthenticationMethodForm(FlaskForm):
+    method_id = IntegerField(validators=[InputRequired()])
+    action = StringField(validators=[InputRequired()])
