@@ -15,6 +15,7 @@ in JSON schema, sampledb schemas can use the following types:
 
 from . import utils
 from .convert_to_schema import convert_to_schema
+from .copy_data import copy_data
 from .generate_placeholder import generate_placeholder
 from .validate_schema import validate_schema
 from .validate import validate
@@ -22,6 +23,7 @@ from .validate import validate
 
 __all__ = [
     'convert_to_schema',
+    'copy_data',
     'generate_placeholder',
     'utils',
     'validate_schema',
