@@ -12,11 +12,17 @@ import re
 from ..errors import ValidationError
 from .utils import units_are_valid
 from .validate import validate
+from .conditions import validate_condition_schema
+from ..languages import get_languages
 
 __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
 
 
-def validate_schema(schema: dict, path: typing.Optional[typing.List[str]] = None) -> None:
+def validate_schema(
+        schema: dict,
+        path: typing.Optional[typing.List[str]] = None,
+        parent_conditions: typing.Optional[typing.List[typing.Tuple]] = None
+) -> None:
     """
     Validates the given schema and raises a ValidationError if it is invalid.
 
@@ -34,8 +40,34 @@ def validate_schema(schema: dict, path: typing.Optional[typing.List[str]] = None
         raise ValidationError('invalid schema (type must be str)', path)
     if 'title' not in schema:
         raise ValidationError('invalid schema (must contain title)', path)
-    if not isinstance(schema['title'], str):
-        raise ValidationError('invalid schema (title must be str)', path)
+    all_language_codes = {
+        language.lang_code
+        for language in get_languages()
+    }
+    if not isinstance(schema['title'], str) and not isinstance(schema['title'], dict):
+        raise ValidationError('title must be str or dict', path)
+    if isinstance(schema['title'], dict):
+        if 'en' not in schema['title']:
+            raise ValidationError('title must include an english translation', path)
+        for lang_code in schema['title'].keys():
+            if lang_code not in all_language_codes:
+                raise ValidationError('title must only contain known languages', path)
+        for note_text in schema['title'].values():
+            if not isinstance(note_text, str):
+                raise ValidationError('title must only contain text', path)
+    if 'style' in schema and not isinstance(schema['style'], str):
+        raise ValidationError('style must only contain text', path)
+    if 'conditions' in schema:
+        if parent_conditions is not None:
+            if not isinstance(schema['conditions'], list):
+                raise ValidationError('conditions must be a list', path)
+            for i, condition in enumerate(schema['conditions']):
+                parent_conditions.append((path + [str(i)], condition))
+        else:
+            raise ValidationError('only schemas with an object parent may contain conditions', path)
+    if not isinstance(schema.get('may_copy', True), bool):
+        raise ValidationError('may_copy must be bool', path)
+
     if path == [] and schema['type'] != 'object':
         raise ValidationError('invalid schema (root must be an object)', path)
     if schema['type'] == 'array':
@@ -68,6 +100,24 @@ def validate_schema(schema: dict, path: typing.Optional[typing.List[str]] = None
         raise ValidationError('invalid type', path)
 
 
+def _validate_note_in_schema(schema: dict, path: typing.List[str]):
+    all_language_codes = {
+        language.lang_code
+        for language in get_languages()
+    }
+    if 'note' in schema and not isinstance(schema['note'], str) and not isinstance(schema['note'], dict):
+        raise ValidationError('note must be str or dict', path)
+    if 'note' in schema and isinstance(schema['note'], dict):
+        if 'en' not in schema['note']:
+            raise ValidationError('note must include an english translation', path)
+        for lang_code in schema['note'].keys():
+            if lang_code not in all_language_codes:
+                raise ValidationError('note must only contain known languages', path)
+        for note_text in schema['note'].values():
+            if not isinstance(note_text, str):
+                raise ValidationError('note must only contain text', path)
+
+
 def _validate_hazards_schema(schema: dict, path: typing.List[str]) -> None:
     """
     Validate the given GHS hazards schema and raise a ValidationError if it is invalid.
@@ -76,7 +126,7 @@ def _validate_hazards_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -89,8 +139,8 @@ def _validate_hazards_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('dataverse_export must be True or False', path)
     if path != ['hazards']:
         raise ValidationError('GHS hazards must be a top-level entry named "hazards"', path)
-    if not isinstance(schema.get('note', ''), str):
-        raise ValidationError('note must be a string', path)
+
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_array_schema(schema: dict, path: typing.List[str]) -> None:
@@ -101,7 +151,7 @@ def _validate_array_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'items', 'style', 'minItems', 'maxItems', 'default'}
+    valid_keys = {'type', 'title', 'items', 'style', 'minItems', 'maxItems', 'defaultItems', 'default', 'may_copy'}
     required_keys = {'type', 'title', 'items'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -126,13 +176,27 @@ def _validate_array_schema(schema: dict, path: typing.List[str]) -> None:
             raise ValidationError('maxItems must not be negative', path)
         else:
             has_max_items = True
+    has_default_items = False
+    if 'defaultItems' in schema:
+        if not isinstance(schema['defaultItems'], int):
+            raise ValidationError('defaultItems must be an integer', path)
+        elif schema['defaultItems'] < 0:
+            raise ValidationError('defaultItems must not be negative', path)
+        else:
+            has_default_items = True
     if has_min_items and has_max_items:
         if schema['minItems'] > schema['maxItems']:
             raise ValidationError('minItems must be less than or equal to maxItems', path)
-    if 'style' in schema and schema['style'] not in ('table', 'horizontal_table', 'list'):
-        raise ValidationError('style must be one of "list", "table" and "horizontal_table"', path)
+    if has_min_items and has_default_items:
+        if schema['minItems'] > schema['defaultItems']:
+            raise ValidationError('minItems must be less than or equal to defaultItems', path)
+    if has_default_items and has_max_items:
+        if schema['defaultItems'] > schema['maxItems']:
+            raise ValidationError('defaultItems must be less than or equal to maxItems', path)
     validate_schema(schema['items'], path + ['[?]'])
     if 'default' in schema:
+        if has_default_items:
+            raise ValidationError('default and defaultItems are mutually exclusive', path)
         validate(schema['default'], schema, path + ['(default)'])
 
 
@@ -144,7 +208,7 @@ def _validate_tags_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'default', 'dataverse_export', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -169,7 +233,7 @@ def _validate_object_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'properties', 'propertyOrder', 'required', 'default'}
+    valid_keys = {'type', 'title', 'properties', 'propertyOrder', 'required', 'default', 'may_copy', 'style'}
     if not path:
         # the top level object may contain a list of properties to be displayed in a table of objects
         valid_keys.add('displayProperties')
@@ -187,10 +251,17 @@ def _validate_object_schema(schema: dict, path: typing.List[str]) -> None:
 
     if not isinstance(schema['properties'], dict):
         raise ValidationError('properties must be dict', path)
+    property_conditions = []
+    property_schemas = {}
     for property_name, property_schema in schema['properties'].items():
         if '__' in property_name:
             raise ValidationError('invalid property name: {}'.format(property_name), path)
-        validate_schema(property_schema, path + [property_name])
+        validate_schema(property_schema, path + [property_name], property_conditions)
+        property_schemas[property_name] = property_schema
+    for path, condition in property_conditions:
+        if not isinstance(condition, dict) or not isinstance(condition.get('type'), str):
+            raise ValidationError('condition must be a dict containg the key type', path)
+        validate_condition_schema(condition, property_schemas, path)
 
     if 'required' in schema:
         if not isinstance(schema['required'], list):
@@ -200,6 +271,8 @@ def _validate_object_schema(schema: dict, path: typing.List[str]) -> None:
                 raise ValidationError('unknown required property: {}'.format(property_name), path)
             if property_name in schema['required'][:i]:
                 raise ValidationError('duplicate required property: {}'.format(property_name), path)
+            if schema['properties'][property_name].get('conditions'):
+                raise ValidationError('conditional required property: {}'.format(property_name), path)
 
     if 'hazards' in schema['properties'] and schema['properties']['hazards']['type'] == 'hazards' and 'hazards' not in schema.get('required', []):
         raise ValidationError('GHS hazards may not be optional', path)
@@ -259,14 +332,41 @@ def _validate_text_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'minLength', 'maxLength', 'choices', 'pattern', 'multiline', 'markdown', 'note', 'placeholder', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'default', 'minLength', 'maxLength', 'choices', 'pattern', 'multiline', 'markdown', 'note', 'placeholder', 'dataverse_export', 'languages', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
         raise ValidationError('unexpected keys in schema: {}'.format(invalid_keys), path)
 
-    if 'default' in schema and not isinstance(schema['default'], str):
-        raise ValidationError('default must be str', path)
+    all_language_codes = {
+        language.lang_code
+        for language in get_languages()
+    }
+    if 'languages' in schema:
+        if schema['languages'] != 'all':
+            if not isinstance(schema['languages'], list):
+                raise ValidationError('languages must be a list of known language codes or "all"', path)
+            for language in schema['languages']:
+                if not isinstance(language, str) or language not in all_language_codes:
+                    raise ValidationError('languages must be a list of known language codes or "all"', path)
+            allowed_language_codes = schema['languages']
+        else:
+            allowed_language_codes = all_language_codes
+    else:
+        allowed_language_codes = {'en'}
+    if 'placeholder' in schema and 'choices' in schema:
+        raise ValidationError('placeholder cannot be used together with choices', path)
+    if 'placeholder' in schema and not isinstance(schema['placeholder'], str) and not isinstance(schema['placeholder'], dict):
+        raise ValidationError('placeholder must be str or dict', path)
+    if 'placeholder' in schema and isinstance(schema['placeholder'], dict):
+        if 'en' not in schema['placeholder']:
+            raise ValidationError('placeholder must include an english translation', path)
+        for lang_code in schema['placeholder'].keys():
+            if lang_code not in all_language_codes:
+                raise ValidationError('placeholder must only contain known languages', path)
+        for placeholder_text in schema['placeholder'].values():
+            if not isinstance(placeholder_text, str):
+                raise ValidationError('placeholder must only contain text', path)
     if 'minLength' in schema and (not isinstance(schema['minLength'], int) or isinstance(schema['minLength'], bool)):
         raise ValidationError('minLength must be int', path)
     if 'maxLength' in schema and (not isinstance(schema['maxLength'], int) or isinstance(schema['maxLength'], bool)):
@@ -282,15 +382,39 @@ def _validate_text_schema(schema: dict, path: typing.List[str]) -> None:
     if 'choices' in schema and not schema['choices']:
         raise ValidationError('choices must not be empty', path)
     if 'choices' in schema:
+        choice_type = None
         for i, choice in enumerate(schema['choices']):
-            if not isinstance(choice, str):
-                raise ValidationError('choice must be str', path + [str(i)])
-            if choice.isspace():
-                raise ValidationError('choice must contain more than whitespace', path + [str(i)])
-    if 'placeholder' in schema and 'choices' in schema:
-        raise ValidationError('placeholder cannot be used together with choices', path)
-    if 'placeholder' in schema and not isinstance(schema['placeholder'], str):
-        raise ValidationError('placeholder must be str', path)
+            if not isinstance(choice, str) and not isinstance(choice, dict):
+                raise ValidationError('choice must be str or dict', path + [str(i)])
+            if choice_type is not None and type(choice) != choice_type:
+                raise ValidationError('choices must be either all str or all dict', path + [str(i)])
+            choice_type = type(choice)
+            if isinstance(choice, dict):
+                if 'en' not in choice:
+                    raise ValidationError('choice must include an english translation', path + [str(i)])
+                for lang_code in choice.keys():
+                    if lang_code not in all_language_codes:
+                        raise ValidationError('choice must only contain known languages', path + [str(i)])
+                for choice_text in choice.values():
+                    if not isinstance(choice_text, str):
+                        raise ValidationError('choice must only contain text', path + [str(i)])
+                    if choice_text.isspace():
+                        raise ValidationError('choice must contain more than whitespace', path + [str(i)])
+            else:
+                if choice.isspace():
+                    raise ValidationError('choice must contain more than whitespace', path + [str(i)])
+    if 'default' in schema and not isinstance(schema['default'], str) and not isinstance(schema['default'], dict):
+        raise ValidationError('default must be str or dict', path)
+    if 'choices' in schema and 'default' in schema:
+        if schema['default'] not in schema['choices']:
+            raise ValidationError('default must only contain a valid choice', path)
+    elif 'default' in schema and isinstance(schema['default'], dict):
+        for lang_code in schema['default'].keys():
+            if lang_code not in allowed_language_codes:
+                raise ValidationError('default must only contain allowed languages', path)
+        for default_text in schema['default'].values():
+            if not isinstance(default_text, str):
+                raise ValidationError('default must only contain text', path)
     if 'pattern' in schema and not isinstance(schema['pattern'], str):
         raise ValidationError('pattern must be str', path)
     if 'pattern' in schema:
@@ -312,8 +436,7 @@ def _validate_text_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('dataverse_export must be True or False', path)
     if 'dataverse_export' in schema and not schema['dataverse_export'] and path == ['name']:
         raise ValidationError('dataverse_export must be True for the object name', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_datetime_schema(schema: dict, path: typing.List[str]) -> None:
@@ -324,7 +447,7 @@ def _validate_datetime_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
@@ -340,8 +463,7 @@ def _validate_datetime_schema(schema: dict, path: typing.List[str]) -> None:
                 raise ValidationError('invalid default value', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
@@ -352,7 +474,7 @@ def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
@@ -362,8 +484,7 @@ def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('default must be bool', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
@@ -374,7 +495,7 @@ def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'units', 'default', 'note', 'placeholder', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'units', 'default', 'note', 'placeholder', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title', 'units'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -384,6 +505,11 @@ def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
 
+    all_language_codes = {
+        language.lang_code
+        for language in get_languages()
+    }
+
     if not isinstance(schema['units'], str):
         raise ValidationError('units must be str', path)
     elif not units_are_valid(schema['units']):
@@ -393,11 +519,18 @@ def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('default must be float or int', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
-
-    if 'placeholder' in schema and not isinstance(schema['placeholder'], str):
-        raise ValidationError('placeholder must be str', path)
+    if 'placeholder' in schema and not isinstance(schema['placeholder'], str) and not isinstance(schema['placeholder'], dict):
+        raise ValidationError('placeholder must be str or dict', path)
+    if 'placeholder' in schema and isinstance(schema['placeholder'], dict):
+        if 'en' not in schema['placeholder']:
+            raise ValidationError('placeholder must include an english translation', path)
+        for lang_code in schema['placeholder'].keys():
+            if lang_code not in all_language_codes:
+                raise ValidationError('placeholder must only contain known languages', path)
+        for placeholder_text in schema['placeholder'].values():
+            if not isinstance(placeholder_text, str):
+                raise ValidationError('placeholder must only contain text', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_sample_schema(schema: dict, path: typing.List[str]) -> None:
@@ -408,7 +541,7 @@ def _validate_sample_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -417,10 +550,10 @@ def _validate_sample_schema(schema: dict, path: typing.List[str]) -> None:
     missing_keys = required_keys - schema_keys
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
+
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_measurement_schema(schema: dict, path: typing.List[str]) -> None:
@@ -431,7 +564,7 @@ def _validate_measurement_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -440,10 +573,10 @@ def _validate_measurement_schema(schema: dict, path: typing.List[str]) -> None:
     missing_keys = required_keys - schema_keys
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
+
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_object_reference_schema(schema: dict, path: typing.List[str]) -> None:
@@ -454,7 +587,7 @@ def _validate_object_reference_schema(schema: dict, path: typing.List[str]) -> N
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'action_type_id', 'action_id', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'note', 'action_type_id', 'action_id', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -463,14 +596,14 @@ def _validate_object_reference_schema(schema: dict, path: typing.List[str]) -> N
     missing_keys = required_keys - schema_keys
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
+
     if 'action_type_id' in schema and not isinstance(schema['action_type_id'], (int, type(None))):
         raise ValidationError('action_type_id must be int or None', path)
     if 'action_id' in schema and not isinstance(schema['action_id'], (int, type(None))):
         raise ValidationError('action_id must be int or None', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_notebook_templates(notebook_templates: typing.Any) -> None:
@@ -531,7 +664,7 @@ def _validate_user_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'default'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'default', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -540,12 +673,11 @@ def _validate_user_schema(schema: dict, path: typing.List[str]) -> None:
     missing_keys = required_keys - schema_keys
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
-    if 'note' in schema and not isinstance(schema['note'], str):
-        raise ValidationError('note must be str', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
     if 'default' in schema and schema['default'] != 'self':
         raise ValidationError('default must be "self"', path)
+    _validate_note_in_schema(schema, path)
 
 
 def _validate_plotly_chart_schema(schema: dict, path: typing.List[str]) -> None:
@@ -556,8 +688,9 @@ def _validate_plotly_chart_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
     schema_keys = schema.keys()
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
         raise ValidationError('unexpected keys in schema: {}'.format(invalid_keys), path)
+    _validate_note_in_schema(schema, path)
