@@ -8,8 +8,11 @@ Usage: python -m sampledb create_action <instrument_id> <type: sample or measure
 import json
 import sys
 from .. import create_app
-from ..logic.actions import create_action, get_action_type, get_action_types
+from ..logic.actions import create_action, get_action_type
+from ..logic.action_type_translations import get_action_types_with_translations_in_language, get_action_type_with_translation_in_language
+from ..logic.action_translations import set_action_translation
 from ..logic.instruments import get_instrument
+from ..logic.languages import Language
 from ..logic.schemas import validate_schema
 from ..logic.errors import InstrumentDoesNotExistError, ValidationError, ActionTypeDoesNotExistError
 from .. import models
@@ -42,11 +45,11 @@ def main(arguments):
         action_type_id = None
     if action_type_id is None:
         print("Error: action type must be one of the following:", file=sys.stderr)
-        print(f'- "sample" (for {get_action_type(models.ActionType.SAMPLE_CREATION).name})', file=sys.stderr)
-        print(f'- "measurement" (for {get_action_type(models.ActionType.MEASUREMENT).name})', file=sys.stderr)
-        print(f'- "simulation" (for {get_action_type(models.ActionType.SIMULATION).name})', file=sys.stderr)
-        for action_type in get_action_types():
-            print(f'- "{action_type.id}" (for {action_type.name})', file=sys.stderr)
+        print(f'- "sample" (for {get_action_type_with_translation_in_language(models.ActionType.SAMPLE_CREATION, Language.ENGLISH).translation.name})', file=sys.stderr)
+        print(f'- "measurement" (for {get_action_type_with_translation_in_language(models.ActionType.MEASUREMENT, Language.ENGLISH).translation.name})', file=sys.stderr)
+        print(f'- "simulation" (for {get_action_type_with_translation_in_language(models.ActionType.SIMULATION, Language.ENGLISH).translation.name})', file=sys.stderr)
+        for action_type in get_action_types_with_translations_in_language(Language.ENGLISH):
+            print(f'- "{action_type.id}" (for {action_type.translation.name})', file=sys.stderr)
         exit(1)
     app = create_app()
     with app.app_context():
@@ -66,8 +69,12 @@ def main(arguments):
         action = create_action(
             instrument_id=instrument_id,
             action_type_id=action_type_id,
-            name=name,
-            description=description,
             schema=schema
         )
+        set_action_translation(
+            language_id=Language.ENGLISH,
+            action_id=action.id,
+            name=name,
+            description=description
+        )
         print("Success: the action has been created in SampleDB (#{})".format(action.id))
